@@ -1,7 +1,11 @@
 <?php
+Co::set([
+    'trace_flags' => SWOOLE_TRACE_HTTP2,
+    'log_level' => 0,
+]);
 $key_dir = dirname(__DIR__) . '/ssl';
-//$http = new swoole_http_server("0.0.0.0", 9501, SWOOLE_BASE);
-$http = new swoole_http_server("0.0.0.0", 9501, SWOOLE_BASE, SWOOLE_SOCK_TCP | SWOOLE_SSL);
+$http = new swoole_http_server("0.0.0.0", 9501, SWOOLE_BASE);
+//$http = new swoole_http_server("0.0.0.0", 9501, SWOOLE_BASE, SWOOLE_SOCK_TCP | SWOOLE_SSL);
 $http->set([
     'open_http2_protocol' => 1,
     'ssl_cert_file' => $key_dir.'/ssl.crt',
