@@ -29,12 +29,12 @@ static const char *method_strings[] =
     "SUBSCRIBE", "UNSUBSCRIBE", "PURGE", "PRI",
 };
 
-int swHttp_get_method(const char *method_str, int method_len)
+int swHttp_get_method(const char *method_str, size_t method_len)
 {
-    int i;
-    for (i = 0; i < SW_HTTP_PRI; i++)
+    int i = 0;
+    for (; i < SW_HTTP_PRI; i++)
     {
-        if (strncasecmp(method_strings[i], method_str, method_len) == 0)
+        if (swoole_strcaseeq(method_strings[i], strlen(method_strings[i]), method_str, method_len))
         {
             return i + 1;
         }
@@ -480,7 +480,7 @@ int swHttpRequest_get_header_info(swHttpRequest *request)
         if (*p == '\n' && *(p-1) == '\r')
         {
             p++;
-            if (strncasecmp(p, SW_STRL("Content-Length:")) == 0)
+            if (SW_STRCASECT(p, pe - p, "Content-Length:"))
             {
                 // strlen("Content-Length:")
                 p += (sizeof("Content-Length:") - 1);
@@ -492,7 +492,7 @@ int swHttpRequest_get_header_info(swHttpRequest *request)
                 request->content_length = atoi(p);
                 got_len = 1;
             }
-            else if (strncasecmp(p, SW_STRL("Connection:")) == 0)
+            else if (SW_STRCASECT(p, pe - p, "Connection:"))
             {
                 // strlen("Connection:")
                 p += (sizeof("Connection:") - 1);
@@ -501,7 +501,7 @@ int swHttpRequest_get_header_info(swHttpRequest *request)
                 {
                     p++;
                 }
-                if (strncasecmp(p, SW_STRL("keep-alive")) == 0)
+                if (SW_STRCASECT(p, pe - p, "keep-alive"))
                 {
                     request->keep_alive = 1;
                 }
@@ -530,10 +530,10 @@ int swHttpRequest_has_expect_header(swHttpRequest *request)
         if (*p == '\r' && pe - p > sizeof("\r\nExpect"))
         {
             p += 2;
-            if (strncasecmp(p, SW_STRL("Expect")) == 0)
+            if (SW_STRCASECT(p, pe - p, "Expect: "))
             {
                 p += sizeof("Expect: ") - 1;
-                if (strncasecmp(p, SW_STRL("100-continue")) == 0)
+                if (SW_STRCASECT(p, pe - p, "100-continue"))
                 {
                     return 1;
                 }
