@@ -23,7 +23,8 @@
 static int swServer_destory(swServer *serv);
 static int swServer_start_check(swServer *serv);
 static void swServer_signal_handler(int sig);
-static void swServer_disable_accept(swReactor *reactor);
+static void swServer_enable_accept(swTimer *timer, swTimer_node *tnode);
+static void swServer_disable_accept(swServer *serv);
 static void swServer_master_update_time(swServer *serv);
 
 static int swServer_tcp_send(swServer *serv, int session_id, void *data, uint32_t length);
@@ -35,10 +36,15 @@ static int swServer_tcp_feedback(swServer *serv, int session_id, int event);
 
 static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, int fd, int server_fd);
 
-static void swServer_disable_accept(swReactor *reactor)
+static void swServer_disable_accept(swServer *serv)
 {
     swListenPort *ls;
-    swServer *serv = (swServer *) reactor->ptr;
+
+    serv->enable_accept_timer = swoole_timer_add(SW_ACCEPT_RETRY_TIME * 1000, 0, swServer_enable_accept, serv);
+    if (serv->enable_accept_timer == nullptr)
+    {
+        return;
+    }
 
     LL_FOREACH(serv->listen_list, ls)
     {
@@ -47,24 +53,25 @@ static void swServer_disable_accept(swReactor *reactor)
         {
             continue;
         }
-        reactor->del(reactor, ls->sock);
+        swoole_event_del(ls->sock);
     }
 }
 
-void swServer_enable_accept(swReactor *reactor)
+static void swServer_enable_accept(swTimer *timer, swTimer_node *tnode)
 {
     swListenPort *ls;
-    swServer *serv = (swServer *) reactor->ptr;
+    swServer *serv = (swServer *) tnode->data;
 
     LL_FOREACH(serv->listen_list, ls)
     {
-        //UDP
-        if (ls->type == SW_SOCK_UDP || ls->type == SW_SOCK_UDP6 || ls->type == SW_SOCK_UNIX_DGRAM)
+        if (swSocket_is_dgram(ls->type))
         {
             continue;
         }
-        reactor->add(reactor, ls->sock, SW_FD_STREAM_SERVER);
+        swoole_event_add(ls->sock, SW_EVENT_READ, SW_FD_STREAM_SERVER);
     }
+
+    serv->enable_accept_timer = nullptr;
 }
 
 void swServer_close_port(swServer *serv, enum swBool_type only_stream_port)
@@ -103,15 +110,14 @@ int swServer_master_onAccept(swReactor *reactor, swEvent *event)
             default:
                 if (errno == EMFILE || errno == ENFILE)
                 {
-                    swServer_disable_accept(reactor);
-                    reactor->disable_accept = 1;
+                    swServer_disable_accept(serv);
                 }
                 swSysWarn("accept() failed");
                 return SW_OK;
             }
         }
 
-        swTrace("[Master] Accept new connection. maxfd=%d|reactor_id=%d|conn=%d", swServer_get_maxfd(serv), reactor->id, new_fd);
+        swTrace("[Master] Accept new connection. maxfd=%d|minfd=%d|reactor_id=%d|conn=%d", swServer_get_maxfd(serv), swServer_get_minfd(serv), reactor->id, new_fd);
 
         //too many connection
         if (new_fd >= (int) serv->max_connection)
@@ -571,7 +577,7 @@ int swServer_start(swServer *serv)
             }
         }
 
-        if (daemon(0, 1) < 0)
+        if (swoole_daemon(0, 1) < 0)
         {
             return SW_ERR;
         }
@@ -720,10 +726,10 @@ void swServer_init(swServer *serv)
     //http server
     serv->http_parse_cookie = 1;
     serv->http_parse_post = 1;
-#ifdef SW_HAVE_ZLIB
+#ifdef SW_HAVE_COMPRESSION
     serv->http_compression = 1;
 #endif
-    serv->http_compression_level = 1; // Z_BEST_SPEED
+    serv->http_compression_level = SW_Z_BEST_SPEED;
     serv->upload_tmp_dir = sw_strdup("/tmp");
 
     serv->buffer_input_size = SW_BUFFER_INPUT_SIZE;
@@ -791,6 +797,25 @@ int swServer_create(swServer *serv)
     }
 }
 
+void swServer_clear_timer(swServer *serv)
+{
+    if (serv->master_timer)
+    {
+        swoole_timer_del(serv->master_timer);
+        serv->master_timer = nullptr;
+    }
+    if (serv->heartbeat_timer)
+    {
+        swoole_timer_del(serv->heartbeat_timer);
+        serv->heartbeat_timer = nullptr;
+    }
+    if (serv->enable_accept_timer)
+    {
+        swoole_timer_del(serv->enable_accept_timer);
+        serv->enable_accept_timer = nullptr;
+    }
+}
+
 int swServer_shutdown(swServer *serv)
 {
     serv->running = 0;
@@ -807,11 +832,7 @@ int swServer_shutdown(swServer *serv)
                 reactor->del(reactor, port->sock);
             }
         }
-        if (serv->master_timer)
-        {
-            swoole_timer_del(serv->master_timer);
-            serv->master_timer = NULL;
-        }
+        swServer_clear_timer(serv);
     }
     else
     {
@@ -1307,21 +1328,38 @@ static int swServer_tcp_close(swServer *serv, int session_id, int reset)
     conn->close_actively = 1;
     swTraceLog(SW_TRACE_CLOSE, "session_id=%d, fd=%d", session_id, conn->session_id);
 
-    int ret;
-    if (!swIsWorker())
+    int retval;
+    swWorker *worker;
+    swDataHead ev = { 0 };
+
+    if (swServer_dispatch_mode_is_mod(serv))
     {
-        swWorker *worker = swServer_get_worker(serv, conn->fd % serv->worker_num);
-        swDataHead ev = {0};
+        int worker_id = swServer_worker_schedule(serv, conn->fd, nullptr);
+        if (worker_id != (int) SwooleWG.id)
+        {
+            worker = swServer_get_worker(serv, worker_id);
+            goto _notify;
+        }
+        else
+        {
+            goto _close;
+        }
+    }
+    else if (!swIsWorker())
+    {
+        worker = swServer_get_worker(serv, conn->fd % serv->worker_num);
+        _notify:
         ev.type = SW_SERVER_EVENT_CLOSE;
         ev.fd = session_id;
         ev.reactor_id = conn->reactor_id;
-        ret = swWorker_send2worker(worker, &ev, sizeof(ev), SW_PIPE_MASTER);
+        retval = swWorker_send2worker(worker, &ev, sizeof(ev), SW_PIPE_MASTER);
     }
     else
     {
-        ret = serv->factory.end(&serv->factory, session_id);
+        _close:
+        retval = serv->factory.end(&serv->factory, session_id);
     }
-    return ret;
+    return retval;
 }
 
 void swServer_signal_init(swServer *serv)
@@ -1790,6 +1828,10 @@ static swConnection* swServer_connection_new(swServer *serv, swListenPort *ls, i
     {
         swServer_set_maxfd(serv, fd);
     }
+    else if (fd < swServer_get_minfd(serv))
+    {
+        swServer_set_minfd(serv, fd);
+    }
 
     connection = &(serv->connection_list[fd]);
     bzero(connection, sizeof(swConnection));
