@@ -304,6 +304,52 @@ static const zend_function_entry swoole_http_client_coro_methods[] =
     PHP_FE_END
 };
 
+void http_parse_set_cookies(const char *at, size_t length, zval *zcookies, zval *zset_cookie_headers)
+{
+    const char *p, *eof = at + length;
+    size_t key_len = 0, value_len = 0;
+    zval zvalue;
+
+    // key
+    p = (char *) memchr(at, '=', length);
+    if (p)
+    {
+        key_len = p - at;
+        p++; // point to value
+    }
+    else
+    {
+        p = at; // key is empty
+    }
+    // value
+    eof = (char*) memchr(p, ';', at + length - p);
+    if (!eof)
+    {
+        eof = at + length;
+    }
+    value_len = eof - p;
+    if (value_len != 0)
+    {
+        ZVAL_STRINGL(&zvalue, p, value_len);
+        Z_STRLEN(zvalue) = php_url_decode(Z_STRVAL(zvalue), value_len);
+    }
+    else
+    {
+        ZVAL_EMPTY_STRING(&zvalue);
+    }
+    if (key_len == 0)
+    {
+        add_next_index_zval(zcookies, &zvalue);
+    }
+    else
+    {
+        add_assoc_zval_ex(zcookies, at, key_len, &zvalue);
+    }
+
+    // set_cookie_headers
+    add_next_index_stringl(zset_cookie_headers, (char *) at, length);
+}
+
 static int http_parser_on_header_field(swoole_http_parser *parser, const char *at, size_t length)
 {
     http_client* http = (http_client*) parser->data;
@@ -319,7 +365,6 @@ static int http_parser_on_header_value(swoole_http_parser *parser, const char *a
     zval *zheaders = sw_zend_read_and_convert_property_array(swoole_http_client_coro_ce, zobject, ZEND_STRL("headers"), 0);
     char *header_name = zend_str_tolower_dup(http->tmp_header_field_name, http->tmp_header_field_name_len);
     size_t header_len = http->tmp_header_field_name_len;
-    int ret = 0;
 
     add_assoc_stringl_ex(zheaders, header_name, http->tmp_header_field_name_len, (char *) at, length);
 
@@ -348,7 +393,7 @@ static int http_parser_on_header_value(swoole_http_parser *parser, const char *a
     {
         zval *zcookies = sw_zend_read_and_convert_property_array(swoole_http_client_coro_ce, zobject, ZEND_STRL("cookies"), 0);
         zval *zset_cookie_headers = sw_zend_read_and_convert_property_array(swoole_http_client_coro_ce, zobject, ZEND_STRL("set_cookie_headers"), 0);
-        ret = http_parse_set_cookies(at, length, zcookies, zset_cookie_headers);
+        http_parse_set_cookies(at, length, zcookies, zset_cookie_headers);
     }
 #ifdef SW_HAVE_COMPRESSION
     else if (SW_STREQ(header_name, header_len, "content-encoding"))
@@ -378,7 +423,7 @@ static int http_parser_on_header_value(swoole_http_parser *parser, const char *a
     }
 
     efree(header_name);
-    return ret;
+    return 0;
 }
 
 static int http_parser_on_headers_complete(swoole_http_parser *parser)
@@ -1440,10 +1485,10 @@ bool http_client::recv_http_response(double timeout)
         swTraceLog(SW_TRACE_HTTP_CLIENT, "parsed_n=%ld, retval=%ld, total_bytes=%ld, completed=%d", parsed_n, retval, total_bytes, parser.state == s_start_res);
         if (parser.state == s_start_res)
         {
-            // websocket stick package
-            if (parser.upgrade && (size_t) retval > parsed_n + 1 + SW_WEBSOCKET_HEADER_LEN)
+            // handle redundant data (websocket packet)
+            if (parser.upgrade && (size_t) retval > parsed_n + SW_WEBSOCKET_HEADER_LEN)
             {
-                swString_sub(buffer, parsed_n + 1, retval - parsed_n - 1);
+                swString_sub(buffer, parsed_n, retval - parsed_n);
             }
             return true;
         }
