@@ -16,6 +16,7 @@
 
 #include "php_swoole_cxx.h"
 #include "swoole_http.h"
+#include "mime_types.h"
 
 extern "C"
 {
@@ -80,6 +81,97 @@ static inline void http_header_key_format(char *key, int length)
     }
 }
 
+static inline swString* http_get_write_buffer(http_context *ctx)
+{
+    if (ctx->co_socket)
+    {
+        swString *buffer = ((Socket *) ctx->private_data)->get_write_buffer();
+        if (buffer != NULL)
+        {
+            return buffer;
+        }
+    }
+    return swoole_http_buffer;
+}
+
+typedef struct
+{
+    http_context *ctx;
+    zend_object std;
+} http_response_t;
+
+static sw_inline http_response_t* php_swoole_http_response_fetch_object(zend_object *obj)
+{
+    return (http_response_t *) ((char *) obj - swoole_http_response_handlers.offset);
+}
+
+http_context * php_swoole_http_response_get_context(zval *zobject)
+{
+    return php_swoole_http_response_fetch_object(Z_OBJ_P(zobject))->ctx;
+}
+
+void php_swoole_http_response_set_context(zval *zobject, http_context *ctx)
+{
+    php_swoole_http_response_fetch_object(Z_OBJ_P(zobject))->ctx = ctx;
+}
+
+static void php_swoole_http_response_free_object(zend_object *object)
+{
+    http_response_t *response = php_swoole_http_response_fetch_object(object);
+    http_context *ctx = response->ctx;
+    zval ztmp; /* bool, not required to release it */
+
+    if (ctx)
+    {
+        if (!ctx->end)
+        {
+            if (ctx->response.status == 0)
+            {
+                ctx->response.status = 500;
+            }
+
+            if (0) { }
+#ifdef SW_USE_HTTP2
+            else if (ctx->stream)
+            {
+                swoole_http2_response_end(ctx, nullptr, &ztmp);
+            }
+#endif
+            else if (ctx->co_socket)
+            {
+                swoole_http_response_end(ctx, nullptr, &ztmp);
+            }
+            else
+            {
+                swServer *serv = (swServer *) ctx->private_data;
+                swConnection *conn = swWorker_get_connection(serv, ctx->fd);
+                if (conn && !conn->closed && !conn->peer_closed && !ctx->detached)
+                {
+#ifdef SW_USE_HTTP2
+                    if (!conn->http2_stream)
+#endif
+                    {
+                        swoole_http_response_end(ctx, nullptr, &ztmp);
+                    }
+                }
+            }
+        }
+        ctx->response.zobject = NULL;
+        swoole_http_context_free(ctx);
+    }
+
+    zend_object_std_dtor(&response->std);
+}
+
+static zend_object *php_swoole_http_response_create_object(zend_class_entry *ce)
+{
+    http_response_t *response = (http_response_t *) ecalloc(1, sizeof(http_response_t) + zend_object_properties_size(ce));
+    zend_object_std_init(&response->std, ce);
+    object_properties_init(&response->std, ce);
+    response->std.handlers = &swoole_http_response_handlers;
+    return &response->std;
+}
+
 static PHP_METHOD(swoole_http_response, write);
 static PHP_METHOD(swoole_http_response, end);
 static PHP_METHOD(swoole_http_response, sendfile);
@@ -159,6 +251,12 @@ ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http_response_create, 0, 0, 1)
     ZEND_ARG_INFO(0, fd)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_swoole_http_response_push, 0, 0, 1)
+    ZEND_ARG_INFO(0, data)
+    ZEND_ARG_INFO(0, opcode)
+    ZEND_ARG_INFO(0, flags)
+ZEND_END_ARG_INFO()
+
 const zend_function_entry swoole_http_response_methods[] =
 {
     PHP_ME(swoole_http_response, initHeader, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
@@ -183,7 +281,7 @@ const zend_function_entry swoole_http_response_methods[] =
      * WebSocket
      */
     PHP_ME(swoole_http_response, upgrade, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
-    PHP_ME(swoole_http_response, push, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
+    PHP_ME(swoole_http_response, push, arginfo_swoole_http_response_push, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_response, recv, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_response, close, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
     PHP_ME(swoole_http_response, __destruct, arginfo_swoole_http_void, ZEND_ACC_PUBLIC)
@@ -196,7 +294,7 @@ void php_swoole_http_response_minit(int module_number)
     SW_SET_CLASS_SERIALIZABLE(swoole_http_response, zend_class_serialize_deny, zend_class_unserialize_deny);
     SW_SET_CLASS_CLONEABLE(swoole_http_response, sw_zend_class_clone_deny);
     SW_SET_CLASS_UNSET_PROPERTY_HANDLER(swoole_http_response, sw_zend_class_unset_property_deny);
-    SW_SET_CLASS_CREATE_WITH_ITS_OWN_HANDLERS(swoole_http_response);
+    SW_SET_CLASS_CUSTOM_OBJECT(swoole_http_response, php_swoole_http_response_create_object, php_swoole_http_response_free_object, http_response_t, std);
 
     zend_declare_property_long(swoole_http_response_ce, ZEND_STRL("fd"), 0,  ZEND_ACC_PUBLIC);
     zend_declare_property_null(swoole_http_response_ce, ZEND_STRL("socket"), ZEND_ACC_PUBLIC);
@@ -215,7 +313,7 @@ static PHP_METHOD(swoole_http_response, write)
         RETURN_FALSE;
     }
 
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
     if (UNEXPECTED(!ctx))
     {
         RETURN_FALSE;
@@ -229,18 +327,20 @@ static PHP_METHOD(swoole_http_response, write)
     }
 #endif
 
-#ifdef SW_HAVE_ZLIB
+#ifdef SW_HAVE_COMPRESSION
     ctx->accept_compression = 0;
 #endif
 
     ctx->private_data_2 = return_value;
 
+    swString *http_buffer = http_get_write_buffer(ctx);
+
     if (!ctx->send_header)
     {
         ctx->chunk = 1;
-        swString_clear(swoole_http_buffer);
-        http_build_header(ctx, swoole_http_buffer, -1);
-        if (!ctx->send(ctx, swoole_http_buffer->str, swoole_http_buffer->length))
+        swString_clear(http_buffer);
+        http_build_header(ctx, http_buffer, -1);
+        if (!ctx->send(ctx, http_buffer->str, http_buffer->length))
         {
             ctx->chunk = 0;
             ctx->send_header = 0;
@@ -267,17 +367,17 @@ static PHP_METHOD(swoole_http_response, write)
     // so the chunk encoding itself is not compressed,
     // **and the data in each chunk is not compressed individually.**
     // The remote endpoint then decodes the stream by concatenating the chunks and decompressing the result.
-    swString_clear(swoole_http_buffer);
+    swString_clear(http_buffer);
     char *hex_string = swoole_dec2hex(http_body.length, 16);
     int hex_len = strlen(hex_string);
     //"%.*s\r\n%.*s\r\n", hex_len, hex_string, body.length, body.str
-    swString_append_ptr(swoole_http_buffer, hex_string, hex_len);
-    swString_append_ptr(swoole_http_buffer, ZEND_STRL("\r\n"));
-    swString_append_ptr(swoole_http_buffer, http_body.str, http_body.length);
-    swString_append_ptr(swoole_http_buffer, ZEND_STRL("\r\n"));
+    swString_append_ptr(http_buffer, hex_string, hex_len);
+    swString_append_ptr(http_buffer, ZEND_STRL("\r\n"));
+    swString_append_ptr(http_buffer, http_body.str, http_body.length);
+    swString_append_ptr(http_buffer, ZEND_STRL("\r\n"));
     sw_free(hex_string);
 
-    RETURN_BOOL(ctx->send(ctx, swoole_http_buffer->str, swoole_http_buffer->length));
+    RETURN_BOOL(ctx->send(ctx, http_buffer->str, http_buffer->length));
 }
 
 static void http_build_header(http_context *ctx, swString *response, int body_length)
@@ -321,27 +421,27 @@ static void http_build_header(http_context *ctx, swString *response, int body_le
             {
                 continue;
             }
-            if (strncasecmp(key, "Server", keylen) == 0)
+            if (SW_STRCASEEQ(key, keylen, "Server"))
             {
                 header_flag |= HTTP_HEADER_SERVER;
             }
-            else if (strncasecmp(key, "Connection", keylen) == 0)
+            else if (SW_STRCASEEQ(key, keylen, "Connection"))
             {
                 header_flag |= HTTP_HEADER_CONNECTION;
             }
-            else if (strncasecmp(key, "Date", keylen) == 0)
+            else if (SW_STRCASEEQ(key, keylen, "Date"))
             {
                 header_flag |= HTTP_HEADER_DATE;
             }
-            else if (strncasecmp(key, "Content-Length", keylen) == 0 && ctx->parser.method != PHP_HTTP_HEAD)
+            else if (SW_STRCASEEQ(key, keylen, "Content-Length") && ctx->parser.method != PHP_HTTP_HEAD)
             {
                 continue; // ignore
             }
-            else if (strncasecmp(key, "Content-Type", keylen) == 0)
+            else if (SW_STRCASEEQ(key, keylen, "Content-Type"))
             {
                 header_flag |= HTTP_HEADER_CONTENT_TYPE;
             }
-            else if (strncasecmp(key, "Transfer-Encoding", keylen) == 0)
+            else if (SW_STRCASEEQ(key, keylen, "Transfer-Encoding"))
             {
                 header_flag |= HTTP_HEADER_TRANSFER_ENCODING;
             }
@@ -400,7 +500,7 @@ static void http_build_header(http_context *ctx, swString *response, int body_le
     // Content-Length
     else if (body_length > 0 || ctx->parser.method != PHP_HTTP_HEAD)
     {
-#ifdef SW_HAVE_ZLIB
+#ifdef SW_HAVE_COMPRESSION
         if (ctx->accept_compression)
         {
             body_length = swoole_zlib_buffer->length;
@@ -427,7 +527,7 @@ static void http_build_header(http_context *ctx, swString *response, int body_le
         }
         SW_HASHTABLE_FOREACH_END();
     }
-#ifdef SW_HAVE_ZLIB
+#ifdef SW_HAVE_COMPRESSION
     //http compress
     if (ctx->accept_compression)
     {
@@ -441,12 +541,17 @@ static void http_build_header(http_context *ctx, swString *response, int body_le
     ctx->send_header = 1;
 }
 
-#ifdef SW_HAVE_ZLIB
+#ifdef SW_HAVE_COMPRESSION
 int swoole_http_response_compress(swString *body, int method, int level)
 {
+#ifdef SW_HAVE_ZLIB
     int encoding;
+#endif
+
+    if (0) { }
+#ifdef SW_HAVE_ZLIB
     //gzip: 0x1f
-    if (method == HTTP_COMPRESS_GZIP)
+    else if (method == HTTP_COMPRESS_GZIP)
     {
         encoding = 0x1f;
     }
@@ -455,6 +560,7 @@ int swoole_http_response_compress(swString *body, int method, int level)
     {
         encoding = -0xf;
     }
+#endif
 #ifdef SW_HAVE_BROTLI
     else if (method == HTTP_COMPRESS_BR)
     {
@@ -477,7 +583,7 @@ int swoole_http_response_compress(swString *body, int method, int level)
         }
 
         size_t input_size = body->length;
-        const uint8_t *input_buffer = (uint8_t *) body->str;
+        const uint8_t *input_buffer = (const uint8_t *) body->str;
         size_t encoded_size = swoole_zlib_buffer->size;
         uint8_t *encoded_buffer = (uint8_t *) swoole_zlib_buffer->str;
 
@@ -501,8 +607,7 @@ int swoole_http_response_compress(swString *body, int method, int level)
         swWarn("Unknown compression method");
         return SW_ERR;
     }
-
-    // ==== ZLIB ====
+#ifdef SW_HAVE_ZLIB
     if (level == Z_NO_COMPRESSION)
     {
         level = Z_DEFAULT_COMPRESSION;
@@ -521,42 +626,41 @@ int swoole_http_response_compress(swString *body, int method, int level)
         }
     }
 
-    z_stream zstream;
-    memset(&zstream, 0, sizeof(zstream));
-
+    z_stream zstream = { 0 };
     int status;
+
     zstream.zalloc = php_zlib_alloc;
     zstream.zfree = php_zlib_free;
 
-    int retval = deflateInit2(&zstream, level, Z_DEFLATED, encoding, MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY);
-
-    if (Z_OK == retval)
+    status = deflateInit2(&zstream, level, Z_DEFLATED, encoding, MAX_MEM_LEVEL, Z_DEFAULT_STRATEGY);
+    if (status != Z_OK)
     {
-        zstream.next_in = (Bytef *) body->str;
-        zstream.next_out = (Bytef *) swoole_zlib_buffer->str;
-        zstream.avail_in = body->length;
-        zstream.avail_out = swoole_zlib_buffer->size;
-
-        status = deflate(&zstream, Z_FINISH);
-        deflateEnd(&zstream);
-
-        if (Z_STREAM_END == status)
-        {
-            swoole_zlib_buffer->length = zstream.total_out;
-            return SW_OK;
-        }
+        swWarn("deflateInit2() failed, Error: [%d]", status);
+        return SW_ERR;
     }
-    else
+
+    zstream.next_in = (Bytef *) body->str;
+    zstream.avail_in = body->length;
+    zstream.next_out = (Bytef *) swoole_zlib_buffer->str;
+    zstream.avail_out = swoole_zlib_buffer->size;
+
+    status = deflate(&zstream, Z_FINISH);
+    deflateEnd(&zstream);
+    if (status != Z_STREAM_END)
     {
-        swWarn("deflateInit2() failed, Error: [%d]", retval);
+        swWarn("deflate() failed, Error: [%d]", status);
+        return SW_ERR;
     }
-    return SW_ERR;
+
+    swoole_zlib_buffer->length = zstream.total_out;
+    return SW_OK;
+#endif
 }
 #endif
 
 static PHP_METHOD(swoole_http_response, initHeader)
 {
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
     if (UNEXPECTED(!ctx))
     {
         RETURN_FALSE;
@@ -572,14 +676,14 @@ static PHP_METHOD(swoole_http_response, initHeader)
 
 static PHP_METHOD(swoole_http_response, end)
 {
-    zval *zdata = NULL;
-
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 1);
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
     if (UNEXPECTED(!ctx))
     {
         RETURN_FALSE;
     }
 
+    zval *zdata = NULL;
+
     ZEND_PARSE_PARAMETERS_START(0, 1)
         Z_PARAM_OPTIONAL
         Z_PARAM_ZVAL_EX(zdata, 1, 0)
@@ -594,10 +698,6 @@ static PHP_METHOD(swoole_http_response, end)
 #endif
     {
         swoole_http_response_end(ctx, zdata, return_value);
-        if (!ctx->upgrade)
-        {
-            swoole_http_context_free(ctx);
-        }
     }
 }
 
@@ -627,8 +727,10 @@ void swoole_http_response_end(http_context *ctx, zval *zdata, zval *return_value
     //no http chunk
     else
     {
-        swString_clear(swoole_http_buffer);
-#ifdef SW_HAVE_ZLIB
+        swString *http_buffer = http_get_write_buffer(ctx);
+
+        swString_clear(http_buffer);
+#ifdef SW_HAVE_COMPRESSION
         if (ctx->accept_compression)
         {
             if (http_body.length == 0 || swoole_http_response_compress(&http_body, ctx->compression_method, ctx->compression_level) != SW_OK)
@@ -637,14 +739,14 @@ void swoole_http_response_end(http_context *ctx, zval *zdata, zval *return_value
             }
         }
 #endif
-        http_build_header(ctx, swoole_http_buffer, http_body.length);
+        http_build_header(ctx, http_buffer, http_body.length);
 
         char *send_body_str;
         size_t send_body_len;
 
         if (http_body.length > 0)
         {
-#ifdef SW_HAVE_ZLIB
+#ifdef SW_HAVE_COMPRESSION
             if (ctx->accept_compression)
             {
                 send_body_str = swoole_zlib_buffer->str;
@@ -663,7 +765,7 @@ void swoole_http_response_end(http_context *ctx, zval *zdata, zval *return_value
             if (send_body_len < SwooleG.pagesize)
 #endif
             {
-                if (swString_append_ptr(swoole_http_buffer, send_body_str, send_body_len) < 0)
+                if (swString_append_ptr(http_buffer, send_body_str, send_body_len) < 0)
                 {
                     ctx->send_header = 0;
                     RETURN_FALSE;
@@ -672,13 +774,14 @@ void swoole_http_response_end(http_context *ctx, zval *zdata, zval *return_value
 #ifdef SW_HTTP_SEND_TWICE
             else
             {
-                if (!ctx->send(ctx, swoole_http_buffer->str, swoole_http_buffer->length))
+                if (!ctx->send(ctx, http_buffer->str, http_buffer->length))
                 {
                     ctx->send_header = 0;
                     RETURN_FALSE;
                 }
                 if (!ctx->send(ctx, send_body_str, send_body_len))
                 {
+                    ctx->end = 1;
                     ctx->close(ctx);
                     RETURN_FALSE;
                 }
@@ -687,9 +790,10 @@ void swoole_http_response_end(http_context *ctx, zval *zdata, zval *return_value
 #endif
         }
 
-        if (!ctx->send(ctx, swoole_http_buffer->str, swoole_http_buffer->length))
+        if (!ctx->send(ctx, http_buffer->str, http_buffer->length))
         {
-            ctx->send_header = 0;
+            ctx->end = 1;
+            ctx->close(ctx);
             RETURN_FALSE;
         }
     }
@@ -775,6 +879,25 @@ bool swoole_http_response_set_header(http_context *ctx, const char *k, size_t kl
 
 static PHP_METHOD(swoole_http_response, sendfile)
 {
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
+    if (UNEXPECTED(!ctx))
+    {
+        RETURN_FALSE;
+    }
+
+    if (ctx->chunk)
+    {
+        php_swoole_fatal_error(E_ERROR, "can't use sendfile when Http-Chunk is enabled");
+        RETURN_FALSE;
+    }
+#ifdef SW_USE_HTTP2
+    if (ctx->stream)
+    {
+        php_swoole_fatal_error(E_WARNING, "can't use sendfile when http2 connection is established");
+        RETURN_FALSE;
+    }
+#endif
+
     char *file;
     size_t l_file;
     zend_long offset = 0;
@@ -784,28 +907,13 @@ static PHP_METHOD(swoole_http_response, sendfile)
     {
         RETURN_FALSE;
     }
-    if (l_file <= 0)
+
+    if (l_file == 0)
     {
         php_swoole_error(E_WARNING, "file name is empty");
         RETURN_FALSE;
     }
 
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
-    if (UNEXPECTED(!ctx))
-    {
-        RETURN_FALSE;
-    }
-
-#ifdef SW_HAVE_ZLIB
-    ctx->accept_compression = 0;
-#endif
-
-    if (ctx->chunk)
-    {
-        php_swoole_fatal_error(E_ERROR, "can't use sendfile when Http-Chunk is enabled");
-        RETURN_FALSE;
-    }
-
     struct stat file_stat;
     if (stat(file, &file_stat) < 0)
     {
@@ -832,28 +940,47 @@ static PHP_METHOD(swoole_http_response, sendfile)
         length = file_stat.st_size - offset;
     }
 
-    swString_clear(swoole_http_buffer);
-    http_build_header(ctx, swoole_http_buffer, length);
+#ifdef SW_HAVE_COMPRESSION
+    ctx->accept_compression = 0;
+#endif
 
-    if (!ctx->send(ctx, swoole_http_buffer->str, swoole_http_buffer->length))
+    if (!ctx->send_header)
     {
-        ctx->send_header = 0;
-        RETURN_FALSE;
+        swString *http_buffer = http_get_write_buffer(ctx);
+
+        swString_clear(http_buffer);
+
+        zval *zheader = sw_zend_read_and_convert_property_array(swoole_http_response_ce, ctx->response.zobject, ZEND_STRL("header"), 0);
+        if (!zend_hash_str_exists(Z_ARRVAL_P(zheader), ZEND_STRL("Content-Type")))
+        {
+            add_assoc_string(zheader, "Content-Type", (char *) swoole_mime_type_get(file));
+        }
+
+        http_build_header(ctx, http_buffer, length);
+
+        if (!ctx->send(ctx, http_buffer->str, http_buffer->length))
+        {
+            ctx->send_header = 0;
+            RETURN_FALSE;
+        }
     }
+
     if (!ctx->sendfile(ctx, file, l_file, offset, length))
     {
-        ctx->send_header = 0;
+        ctx->close(ctx);
         RETURN_FALSE;
     }
+
     if (!ctx->keepalive)
     {
         ctx->close(ctx);
     }
-    swoole_http_context_free(ctx);
+
+    ctx->end = 1;
     RETURN_TRUE;
 }
 
-static void swoole_http_response_cookie(INTERNAL_FUNCTION_PARAMETERS, const bool url_encode)
+static void php_swoole_http_response_cookie(INTERNAL_FUNCTION_PARAMETERS, const bool url_encode)
 {
     char *name, *value = NULL, *path = NULL, *domain = NULL, *samesite = NULL;
     zend_long expires = 0;
@@ -872,7 +999,7 @@ static void swoole_http_response_cookie(INTERNAL_FUNCTION_PARAMETERS, const bool
         Z_PARAM_STRING(samesite, samesite_len)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
     if (UNEXPECTED(!ctx))
     {
         RETURN_FALSE;
@@ -960,12 +1087,12 @@ static void swoole_http_response_cookie(INTERNAL_FUNCTION_PARAMETERS, const bool
 
 static PHP_METHOD(swoole_http_response, cookie)
 {
-    swoole_http_response_cookie(INTERNAL_FUNCTION_PARAM_PASSTHRU, true);
+    php_swoole_http_response_cookie(INTERNAL_FUNCTION_PARAM_PASSTHRU, true);
 }
 
 static PHP_METHOD(swoole_http_response, rawcookie)
 {
-    swoole_http_response_cookie(INTERNAL_FUNCTION_PARAM_PASSTHRU, false);
+    php_swoole_http_response_cookie(INTERNAL_FUNCTION_PARAM_PASSTHRU, false);
 }
 
 static PHP_METHOD(swoole_http_response, status)
@@ -980,7 +1107,7 @@ static PHP_METHOD(swoole_http_response, status)
         Z_PARAM_STRING(reason, reason_len)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
     if (UNEXPECTED(!ctx))
     {
         RETURN_FALSE;
@@ -1004,7 +1131,7 @@ static PHP_METHOD(swoole_http_response, header)
         Z_PARAM_BOOL(ucwords)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
     if (UNEXPECTED(!ctx))
     {
         RETURN_FALSE;
@@ -1025,7 +1152,7 @@ static PHP_METHOD(swoole_http_response, trailer)
         Z_PARAM_STRING_EX(v, vlen, 1, 0)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
     if (!ctx || !ctx->stream)
     {
         RETURN_FALSE;
@@ -1056,57 +1183,90 @@ static PHP_METHOD(swoole_http_response, trailer)
 
 static PHP_METHOD(swoole_http_response, ping)
 {
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
-    if (!ctx || !ctx->stream)
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
+    if (UNEXPECTED(!ctx))
     {
         RETURN_FALSE;
     }
+    if (UNEXPECTED(!ctx->stream))
+    {
+        php_swoole_fatal_error(E_WARNING, "fd[%d] is not a HTTP2 conncetion", ctx->fd);
+        RETURN_FALSE;
+    }
     SW_CHECK_RETURN(swoole_http2_server_ping(ctx));
 }
 #endif
 
 static PHP_METHOD(swoole_http_response, upgrade)
 {
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
-    if (UNEXPECTED(!ctx) || !ctx->co_socket)
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
+    if (UNEXPECTED(!ctx))
     {
         RETURN_FALSE;
     }
+    if (UNEXPECTED(!ctx->co_socket))
+    {
+        php_swoole_fatal_error(E_WARNING, "async server dose not support protocol upgrade");
+        RETURN_FALSE;
+    }
     RETVAL_BOOL(swoole_websocket_handshake(ctx));
 }
 
 static PHP_METHOD(swoole_http_response, push)
 {
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
-    if (UNEXPECTED(!ctx) || !ctx->co_socket || !ctx->upgrade)
+    http_context *ctx = php_swoole_http_response_get_context(ZEND_THIS);
+    if (UNEXPECTED(!ctx))
     {
-        SwooleG.error = SW_ERROR_CLIENT_NO_CONNECTION;
+        SwooleG.error = SW_ERROR_SESSION_CLOSED;
+        RETURN_FALSE;
+    }
+    if (UNEXPECTED(!ctx->co_socket || !ctx->upgrade))
+    {
+        php_swoole_fatal_error(E_WARNING, "fd[%d] is not a websocket conncetion", ctx->fd);
         RETURN_FALSE;
     }
 
-    zval *zdata = NULL;
+    zval *zdata;
     zend_long opcode = WEBSOCKET_OPCODE_TEXT;
-    zend_bool finished = 1;
+    zval *zflags = NULL;
+    zend_long flags = SW_WEBSOCKET_FLAG_FIN;
 
-    if (zend_parse_parameters(ZEND_NUM_ARGS(), "z|lb", &zdata, &opcode, &finished) == FAILURE)
+    ZEND_PARSE_PARAMETERS_START(1, 3)
+        Z_PARAM_ZVAL(zdata)
+        Z_PARAM_OPTIONAL
+        Z_PARAM_LONG(opcode)
+        Z_PARAM_ZVAL_EX(zflags, 1, 0)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
+    if (zflags != NULL)
     {
-        RETURN_FALSE;
+        flags = zval_get_long(zflags);
     }
 
-    swString_clear(swoole_http_buffer);
-    if (php_swoole_websocket_frame_pack(swoole_http_buffer, zdata, opcode, finished, 0) < 0)
+    swString *http_buffer = http_get_write_buffer(ctx);
+    swString_clear(http_buffer);
+    if (php_swoole_websocket_frame_is_object(zdata))
     {
-        RETURN_FALSE;
+        if (php_swoole_websocket_frame_object_pack(http_buffer, zdata, 0, ctx->websocket_compression) < 0)
+        {
+            RETURN_FALSE;
+        }
     }
-    RETURN_BOOL(ctx->send(ctx, swoole_http_buffer->str, swoole_http_buffer->length));
+    else
+    {
+        if (php_swoole_websocket_frame_pack(http_buffer, zdata, opcode, flags & SW_WEBSOCKET_FLAGS_ALL, 0, ctx->websocket_compression) < 0)
+        {
+            RETURN_FALSE;
+        }
+    }
+    RETURN_BOOL(ctx->send(ctx, http_buffer->str, http_buffer->length));
 }
 
 static PHP_METHOD(swoole_http_response, close)
 {
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
-    if (UNEXPECTED(!ctx) || !ctx->co_socket || !ctx->upgrade)
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
+    if (UNEXPECTED(!ctx))
     {
-        SwooleG.error = SW_ERROR_CLIENT_NO_CONNECTION;
         RETURN_FALSE;
     }
     RETURN_BOOL(ctx->close(ctx));
@@ -1114,15 +1274,27 @@ static PHP_METHOD(swoole_http_response, close)
 
 static PHP_METHOD(swoole_http_response, recv)
 {
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
-    if (UNEXPECTED(!ctx) || !ctx->co_socket || !ctx->upgrade)
+    http_context *ctx = php_swoole_http_response_get_context(ZEND_THIS);
+    if (UNEXPECTED(!ctx))
     {
-        SwooleG.error = SW_ERROR_CLIENT_NO_CONNECTION;
+        SwooleG.error = SW_ERROR_SESSION_CLOSED;
+        RETURN_FALSE;
+    }
+    if (UNEXPECTED(!ctx->co_socket || !ctx->upgrade))
+    {
+        php_swoole_fatal_error(E_WARNING, "fd[%d] is not a websocket conncetion", ctx->fd);
         RETURN_FALSE;
     }
 
+    double timeout = 0;
+
+    ZEND_PARSE_PARAMETERS_START_EX(ZEND_PARSE_PARAMS_THROW, 0, 1)
+        Z_PARAM_OPTIONAL
+        Z_PARAM_DOUBLE(timeout)
+    ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
+
     Socket *sock = (Socket *) ctx->private_data;
-    ssize_t retval = sock->recv_packet(0);
+    ssize_t retval = sock->recv_packet(timeout);
     swString _tmp;
 
     if (retval < 0)
@@ -1138,23 +1310,34 @@ static PHP_METHOD(swoole_http_response, recv)
     {
         _tmp.str = sock->get_read_buffer()->str;
         _tmp.length = retval;
+
+#ifdef SW_HAVE_ZLIB
+        php_swoole_websocket_frame_unpack_ex(&_tmp, return_value, ctx->websocket_compression);
+#else
         php_swoole_websocket_frame_unpack(&_tmp, return_value);
+#endif
     }
 }
 
 static PHP_METHOD(swoole_http_response, detach)
 {
-    http_context *context = swoole_http_context_get(ZEND_THIS, 0);
-    if (!context)
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
+    if (!ctx)
     {
         RETURN_FALSE;
     }
-    context->detached = 1;
+    ctx->detached = 1;
     RETURN_TRUE;
 }
 
 static PHP_METHOD(swoole_http_response, create)
 {
+    if (!SwooleG.serv || !SwooleG.serv->gs->start)
+    {
+        php_swoole_fatal_error(E_WARNING, "server is not running");
+        RETURN_FALSE;
+    }
+
     zend_long fd;
 
     ZEND_PARSE_PARAMETERS_START(1, 1)
@@ -1168,16 +1351,12 @@ static PHP_METHOD(swoole_http_response, create)
         RETURN_FALSE;
     }
     ctx->fd = (int) fd;
-
-    if (!SwooleG.serv)
-    {
-        RETURN_FALSE;
-    }
+    ctx->keepalive = 1;
 
     swoole_http_server_init_context(SwooleG.serv, ctx);
 
     object_init_ex(return_value, swoole_http_response_ce);
-    swoole_set_object(return_value, ctx);
+    php_swoole_http_response_set_context(return_value, ctx);
     ctx->response.zobject = return_value;
     sw_copy_to_stack(ctx->response.zobject, ctx->response._zobject);
 
@@ -1195,7 +1374,7 @@ static PHP_METHOD(swoole_http_response, redirect)
         Z_PARAM_ZVAL_EX(zhttp_code, 1, 0)
     ZEND_PARSE_PARAMETERS_END_EX(RETURN_FALSE);
 
-    http_context *ctx = swoole_http_context_get(ZEND_THIS, 0);
+    http_context *ctx = php_swoole_http_response_get_and_check_context(ZEND_THIS);
     if (UNEXPECTED(!ctx))
     {
         RETURN_FALSE;
@@ -1220,47 +1399,6 @@ static PHP_METHOD(swoole_http_response, redirect)
         return;
     }
     swoole_http_response_end(ctx, nullptr, return_value);
-    if (!ctx->end)
-    {
-        swoole_http_context_free(ctx);
-    }
 }
 
-static PHP_METHOD(swoole_http_response, __destruct)
-{
-    SW_PREVENT_USER_DESTRUCT();
-
-    http_context *ctx = (http_context *) swoole_get_object(ZEND_THIS);
-    if (ctx)
-    {
-        if (!ctx->end)
-        {
-            if (ctx->response.status == 0)
-            {
-                ctx->response.status = 500;
-            }
-            if (ctx->co_socket)
-            {
-                swoole_http_response_end(ctx, nullptr, return_value);
-            }
-            else
-            {
-                swServer *serv = (swServer *) ctx->private_data;
-                swConnection *conn = swWorker_get_connection(serv, ctx->fd);
-                if (!conn || conn->closed || conn->peer_closed || ctx->detached)
-                {
-                    swoole_http_context_free(ctx);
-                    ctx = nullptr;
-                }
-                else
-                {
-                    swoole_http_response_end(ctx, nullptr, return_value);
-                }
-            }
-        }
-        if (ctx)
-        {
-            swoole_http_context_free(ctx);
-        }
-    }
-}
+static PHP_METHOD(swoole_http_response, __destruct) { }
